#requires -version 2.0

# Jeffery Hicks
# http://jdhitsolutions.com/blog
# follow on Twitter: http://twitter.com/JeffHicks
#
# "Those who forget to script are doomed to repeat their work."

#  ****************************************************************
#  * DO NOT USE IN A PRODUCTION ENVIRONMENT UNTIL YOU HAVE TESTED *
#  * THOROUGHLY IN A LAB ENVIRONMENT. USE AT YOUR OWN RISK.  IF   *
#  * YOU DO NOT UNDERSTAND WHAT THIS SCRIPT DOES OR HOW IT WORKS, *
#  * DO NOT USE IT OUTSIDE OF A SECURE, TEST SETTING.             *
#  ****************************************************************

Function Backup-WindowsUpdateLog {
<#
.Synopsis
Backup the Windows Update log
.Description
Create a backup copy of the Windows Update log on the specified computer. You can specify a path 
for the backup file that is relative to the remote computer. Otherwise the default is created in 
the Windows folder. The file will be named using the format: 

WindowsUpdate-Computername_yyyyMMddmm.log

This should be run in an elevated session.

.Parameter Computername
The name of the computer to connect to. The default is the local computer. This function uses WinRM 
to manage the log on remote computers.
.Parameter Path
The path for the backup copy. This path
is relative to the remote computer. Any existing files with the same name will be overwritten. The 
default path is the %WINDIR% folder.
.Example
PS C:\> Backup-WindowsUpdateLog

Backup the local Windows Update log to C:\Windows
.Example
PS C:\> Backup-WindowsUpdateLog -computername serenity,quark -verbose -path "\\server01\backups"

Backup the Windows Update logs for Serenity and Quark to a UNC.
.Link
Clear-WindowsUpdateLog
Get-WindowsUpdateLog
.Inputs
Strings for computernames
.Outputs
File object

.Notes
 NAME:      Backup-WindowsUpdateLog
 VERSION:   1.1
 AUTHOR:    Jeffery Hicks http://jdhitsolutions.com/blog
 LASTEDIT:  8/3/2010
 
 Learn more with a copy of Windows PowerShell 2.0: TFM (SAPIEN Press 2010)
#>

[cmdletBinding()]

Param(
    [Parameter(Position=0,ValueFromPipeline=$True)]
    [string[]]$Computername=$env:computername,
    [string]$Path=$env:Windir
)

    Begin {
        Write-Verbose "Starting $($myinvocation.mycommand)"
        $log="$env:windir\WindowsUpdate.log"
        write-verbose "Backing up to $path"
    }

    Process {
        foreach ($computer in $computername) {
            #test connection
            Write-Verbose "Testing $($computer.toUpper())"
            if (Test-Connection -ComputerName $computer -quiet) {
            
                #define a scriptblock
                $sb={
                    Param([string]$log,[string]$path)
                    #define the backup copy filename and path
                    $filecopy=("WindowsUpdate-{0}_{1}.log" -f $env:computername,(get-date -format "yyyyMMddmm"))
                    $copy=Join-path -Path $path -ChildPath $filecopy
                    write-host "$env:computername : Copying $log to $copy" -ForegroundColor Green
                    Copy-Item -Path $log -Destination $copy -PassThru
                }    
                                                
                #if remote then use invoke-command
                if ($computer -eq $env:computername) {
                        Write-Verbose "Executing locally"
                        &$sb $log $path
                 }
                   else {
                        Write-Verbose "Executing remotely: $($computer.ToUpper())"
                        Invoke-Command -ScriptBlock $sb -ComputerName $computer -ArgumentList @($log,$path)
                    }
            }
            else {
                Write-Warning "Failed to connect to $($computer.ToUpper())"
            }
        
        } #foreach
    } #process

    End {
        Write-Verbose "Ending $($myinvocation.mycommand)"
    }

} #end function


