#requires -version 2.0

# Jeffery Hicks
# http://jdhitsolutions.com/blog
# follow on Twitter: http://twitter.com/JeffHicks
#
# "Those who forget to script are doomed to repeat their work."

#  ****************************************************************
#  * DO NOT USE IN A PRODUCTION ENVIRONMENT UNTIL YOU HAVE TESTED *
#  * THOROUGHLY IN A LAB ENVIRONMENT. USE AT YOUR OWN RISK.  IF   *
#  * YOU DO NOT UNDERSTAND WHAT THIS SCRIPT DOES OR HOW IT WORKS, *
#  * DO NOT USE IT OUTSIDE OF A SECURE, TEST SETTING.             *
#  ****************************************************************


Function Get-WindowsUpdate {

<#
.Synopsis
Get Windows Updates
.Description
Query the Windows Update Server and return a collection of available updates. The default is to
select all, but you can also query recommended updates only.

.Parameter All
Get all available updates. This is the default.
.Parameter Recommended
Get only recommended updates
.Example
PS C:\> get-windowsupdate

Title            : Update for Windows 7 for x64-based Systems (KB981793)
Description      : Install this update to resolve issues caused by revised daylight saving time and time zone laws in severa
                   l countries. This update enables your computer to automatically adjust the computer clock on the correct 
                   date in 2010. After you install this item, you may have to restart your computer.
IsPresent        : False
RebootRequired   : False
Identity         : System.__ComObject
IsDownloaded     : True
KB               : 981793
Severity         : None
SecurityBulletin : {}
Category         : {Update Rollups, Windows 7}
SizeKB           : 502
MoreInfo         : http://support.microsoft.com/KB/981793
.Example
PS C:\Scripts> get-windowsupdate -recommendedOnly | Select Title,Severity

Title                                        Severity                                    
-----                                        --------                                    
Update for Windows 7 for x64-based System... None                                        
Security Update for Windows 7 for x64-bas... Important                                   
Microsoft .NET Framework 3.5 SP1 Security... Important                                   
Windows Malicious Software Removal Tool x... None                                        
Security Update for Windows 7 for x64-bas... Important     
.Link
http://msdn.microsoft.com/en-us/library/aa387287%28v=VS.85%29.aspx
.Inputs
None
.Outputs
custom Windows Update object
.Notes
 NAME:      Get-WindowsUpdate
 VERSION:   1.1
 AUTHOR:    Jeffery Hicks http://jdhitsolutions.com/blog
 LASTEDIT:  8/3/2010
 
 Learn more with a copy of Windows PowerShell 2.0: TFM (SAPIEN Press 2010)

#>

[cmdletbinding(DefaultParameterSetName="All")]

Param(
    [Parameter(ParameterSetName="All")]
    [switch]$All,
    [Parameter(ParameterSetName="Recommended")]
    [switch]$RecommendedOnly
)

Write-Verbose "Starting $($myinvocation.mycommand)"
#create session objects
$updateSession = New-Object -ComObject "Microsoft.Update.Session"
$updateSearcher = $updateSession.CreateUpdateSearcher()

#search for specified updates
if ($recommendedOnly) {
    write-verbose "Searching for recommended updates"
     $results=$updateSearcher.Search("IsInstalled=0 and Type='Software' and AutoSelectOnWebsites=1")
}
else {
    Write-Verbose "Searching for all updates"
    $results=$updateSearcher.Search("IsInstalled=0 and Type='Software'")
}

Write-Verbose "Found $($results.updates.count) updates"
$updates=$results.updates | Select-object -property Title,Description,IsPresent,`
RebootRequired,Identity,IsDownloaded,`
@{Name="KB";Expression={$_.KBArticleIDS}},`
@{Name="Severity";Expression={
 if (-not $_.MsrcSeverity) {
    #no severity found so set the value to None
    "None"
 }
 else {
    #otherwise use the specified severity
    $_.MsrcSeverity
 }}},`
@{Name="SecurityBulletin";Expression={$_.SecurityBulletinIDs}},`
@{Name="Category";Expression={
  $categories=@()
  #add each category name to $categories
  foreach ($item in $_.categories) { 
     $categories+=$item.name
  }
  #write to pipeline
  $categories
}},`
@{Name="SizeKB";Expression={$_.MaxDownLoadSize/1KB -as [int]  }},`
@{Name="MoreInfo";Expression={$_.MoreInfoUrls }}

if ($updates) {
    $updates
}
else {
   Write-Host "No updates found." -ForegroundColor Magenta
}

Write-Verbose "Ending $($myinvocation.mycommand)"

} #end Function



