#requires -version 2.0

# Jeffery Hicks
# http://jdhitsolutions.com/blog
# follow on Twitter: http://twitter.com/JeffHicks
#
# "Those who forget to script are doomed to repeat their work."

#  ****************************************************************
#  * DO NOT USE IN A PRODUCTION ENVIRONMENT UNTIL YOU HAVE TESTED *
#  * THOROUGHLY IN A LAB ENVIRONMENT. USE AT YOUR OWN RISK.  IF   *
#  * YOU DO NOT UNDERSTAND WHAT THIS SCRIPT DOES OR HOW IT WORKS, *
#  * DO NOT USE IT OUTSIDE OF A SECURE, TEST SETTING.             *
#  ****************************************************************

Function Get-WindowsUpdateLog  {
<#
.Synopsis
Retrieve Windows Update log.
.Description
Convert the windows update log to objects that can be sorted, grouped, filtered or whatever.  The 
date and time values are combined to a single datetime value.  Messages that are all # characters 
are skipped. Characters #,+ and * are stripped out of the message data.

This script does not use WinRM or WMI for the remote connection. Instead it relies on accessing the 
default administrative share:

\\computername\admin$

You can also use the command to get a backed up Windows Update log.
.Parameter Path
The file name and path to the Windows Update log or a backup. The default is
\\computername\admin$\WindowsUpdate.log

.Parameter Computername
The name of the computer to connect to. The default is the local computer. This script does not use 
WinRM or WMI for the remote connection. Instead it relies on accessing the default administrative 
share:

\\computername\admin$

It also uses this remote connection for the local computer as well.

.Parameter ShowProgress
Use Write-Progress to show progress. CAUTION: This will slow down overall processing.
.Parameter AsJob
Create a background job for this task. The default job name is Get-WU. Even when querying a remote 
computer, the job is always created locally.
.Example
PS C:\> $wu=get-windowsupdatelog -verbose
VERBOSE: Starting Get-WindowsUpdateLog
VERBOSE: Retrieved 17895 lines
VERBOSE: Temp file is C:\Users\Jeff\AppData\Local\Temp\tmpEA83.tmp
VERBOSE: Initial import 17895 items
VERBOSE: Cleaned import 16265 items
VERBOSE: Creating new objects
VERBOSE: Removing C:\Users\Jeff\AppData\Local\Temp\tmpEA83.tmp
VERBOSE: Ending Get-WindowsUpdateLog
PS C:\> $wu | where {$_.datetime -gt (get-date).date} | select datetime,message | ft -auto -wrap

Import all windows update events and then only display messages from the current date.
.Example
PS C:\> get-windowsupdatelog -asjob

Id              Name            State      HasMoreData     Location             Command
--              ----            -----      -----------     --------             -------
31              Get-WU          Running    True            localhost            ...

PS C:\> $wu=Receive-Job -id 31 -keep | select-object -property * -ExcludeProperty RunspaceID
PS C:\> $wu[0]


Component : AU
Message   : AU finished delayed initialization
PID       : 1092
TID       : c48
Category  : Normal
Datetime  : 4/27/2010 7:17:32 AM

Get Windows Update log through a background job then store results in a variable, excluding
the runspaceID property from the job.
.Example
PS C:\> $log=get-windowsupdatelog -path C:\windows\WindowsUpdate-SERENITY_2010080350.log

Get a backup Windows Update log.
.Link
http://support.microsoft.com/kb/902093
.Link
Clear-WindowsUpdateLog
Backup-WindowsUpdateLog
.Inputs
Strings for computernames
.Outputs
custom object
.Notes
 NAME:      Get-WindowsUpdateLog
 VERSION:   2.1
 AUTHOR:    Jeffery Hicks http://jdhitsolutions.com/blog
 LASTEDIT:  8/3/2010
 
 Learn more with a copy of Windows PowerShell 2.0: TFM (SAPIEN Press 2010)
#>

[cmdletbinding()]

Param (
[Parameter(Position=0,ValueFromPipeline=$true)]
[string[]]$computername=$env:computername,
[string]$path,
[switch]$ShowProgress,
[switch]$AsJob
)

Begin {
    Write-Verbose "Starting $($myinvocation.mycommand)"
}

Process {

foreach ($computer in $computername) {
    
    #if an alternate file was specified, use it.
    if ($path) {
        $file=$path
    }
    else {
        $file="\\$computer\admin$\windowsupdate.log"
    }
    
    #verify access to the log file
    Write-Verbose "Getting $file"
    
    Try {
        #throw an exception if the log file cannot be found
        If (-Not (Test-path $file)) {
            Throw
        }
        else {
            $ok=$true
        }
    }

    Catch {
        Write-Warning "Failed to find $file."
    }

    if ($ok) {
     #define the scriptblock
     $sb={
        Param([string]$file,[string]$computername)
        
        #get contents of WindowsUpdate.log and convert to CSV objects
        #Thanks to Shay Levy for this performance tweak
        $import=Get-Content $file | 
        ConvertFrom-Csv -Delimiter "`t" -Header Date,Time,PID,TID,Component,Message | 
        Where-Object {$_.Message -match "\w"}
        
        $i=0
        Write-Verbose "Creating new objects"
        $import | foreach {
            $i++
            if ($showProgress) {
            Write-Progress -Activity "Get-WindowsUpdateLog" `
                -Status "Importing items" `
                -currentoperation $i
            }
            #strip off milliseconds from time
            $time=$_.time.Substring(0,$_.time.lastindexof(":"))
            
            #strip off *,# and + characters and trim message
            [regex]$r="[\*|\+|#]"
            $message=($r.replace($_.message,"")).Trim()
            
            #build a new datetime
            [datetime]$dt=("{0} {1}" -f $_.date,$time)
            
            #Categorize the message
            Switch -regex ($message) {
                "^FATAL:"        {$category="Fatal"}
                "^WARNING:"      {$category="Warning"}
                "^REPORT EVENT:" {$category="Report"}
                Default          {$category="Normal"}
            }
            
            #write a new custom object
            New-Object PSObject -Property @{
                Datetime=$dt
                PID=$_.pid
                TID=$_.tid
                Component=$_.component
                Message=$message
                Category=$category
                Computername=$computername.toUpper()
            }
      
        } #end foreach
        If ($showProgress) {
            Write-Progress -Activity $($myinvocation.mycommand) `
            -Status "Done!" -completed
         }
       
         
        } #end $sb
        
        if ($AsJob) {
            Write-Verbose "Creating job"
            Start-Job -Name "Get-WU" -ScriptBlock $sb -ArgumentList @($file,$computername)
        }
        else {
            &$sb $file $computername
        }
        
      } #end if $data
 } #foreach computer
} #process

End {
    Write-Verbose "Ending $($myinvocation.mycommand)"
 }
} #end function

