#requires -version 2.0

# Jeffery Hicks
# http://jdhitsolutions.com/blog
# follow on Twitter: http://twitter.com/JeffHicks
#
# "Those who forget to script are doomed to repeat their work."

#  ****************************************************************
#  * DO NOT USE IN A PRODUCTION ENVIRONMENT UNTIL YOU HAVE TESTED *
#  * THOROUGHLY IN A LAB ENVIRONMENT. USE AT YOUR OWN RISK.  IF   *
#  * YOU DO NOT UNDERSTAND WHAT THIS SCRIPT DOES OR HOW IT WORKS, *
#  * DO NOT USE IT OUTSIDE OF A SECURE, TEST SETTING.             *
#  ****************************************************************

Function Install-WindowsUpdate {

<#
.Synopsis
Install one or more Windows Update patches.
.Description
Download and install all or specific Windows Updates. The default action is to install critical 
updates only. Depending on your Windows Update configuration you may need to run this command in 
an elevated session.

Installing Windows Updates must be done via a local session. This function will not work remotely. 
Although you could set up a scheduled task remotely to run it.

This function supports -WhatIf.
.Parameter Title
The update title.
.Parameter All
Install all available updates. 
.Parameter Severity
Install only updates that match a severity level of either Critical or Important. 
The default value is Critical.
.Parameter Reboot
Restart the computer automatically if an update requires it.
.Parameter Force
When used with Reboot, will force a reboot, closing all open applications and logging off
users.
.Example
PS C:\> Install-WindowsUpdate -all

Install any and all Windows updates.
.Example
PS C:\> Install-WindowsUpdate -severity critical -reboot -Force
RebootRequired : True
InstallDate    : 6/17/2010 2:54:28 PM
Result         : Success
Computername   : SERVER01
Severity       : Critical
Title          : Cumulative Security Update for ActiveX Killbits for Windows Server 2008 R2 x64 Edition (KB980195)

RebootRequired : False
InstallDate    : 6/17/2010 2:54:36 PM
Result         : Success
Computername   : SERVER01
Severity       : Critical
Title          : Security Update for Windows Server 2008 R2 x64 Edition (KB979482)
.Example
PS C:\> $t="Security Update for Windows Server 2008 R2 x64 Edition (KB979559)"
PS C:\> install-windowsupdate $t

RebootRequired : True
InstallDate    : 6/17/2010 2:51:35 PM
Result         : Success
Computername   : SERVER01
Severity       : Important
Title          : Security Update for Windows Server 2008 R2 x64 Edition (KB979559)

One or more updates requires a reboot.

Install critical updates and reboot if required.
.Link
http://msdn.microsoft.com/en-us/library/aa387287%28v=VS.85%29.aspx
.Inputs
Strings for update titles
.Outputs
Custom object
.Notes
 NAME:      Install-WindowsUpdate
 VERSION:   2.0
 AUTHOR:    Jeffery Hicks http://jdhitsolutions.com/blog
 LASTEDIT:  8/3/2010
 
 Learn more with a copy of Windows PowerShell 2.0: TFM (SAPIEN Press 2010)
#>

[cmdletbinding(SupportsShouldProcess=$True,DefaultParameterSetName="Severity")]

Param(
[Parameter(Position=0,ValueFromPipeline=$True,
ValueFromPipelinebyPropertyName=$True,ParameterSetName="Title")]
[string[]]$Title,
[Parameter(ValueFromPipeline=$False,ParameterSetName="All")]
[switch]$All,
[Parameter(ValueFromPipeline=$False,ParameterSetName="Severity")]
[ValidateSet("Important","Critical")]
[string]$Severity="Critical",
[switch]$Reboot,
[switch]$Force

)

Begin {
    Write-Verbose "Starting $($myinvocation.mycommand)"
    
     if ($title) {
        Write-Verbose "Processing by title"
    }
    
    #create session objects
    $updateSession = New-Object -ComObject "Microsoft.Update.Session"
    $updateSearcher = $updateSession.CreateupdateSearcher()
    $updatesToDownload = New-Object -ComObject "Microsoft.Update.UpdateColl"
    $updatesToInstall = New-Object -ComObject "Microsoft.Update.UpdateColl"
    $downloader = $updateSession.CreateUpdateDownloader() 
    $installer = $updateSession.CreateUpdateInstaller()
    
    #get the collection of available updates
    Write-Verbose "Retrieving available updates"
    $results=$updateSearcher.Search("IsInstalled=0 and Type='Software'")
    Write-Verbose "Retrieved $($results.updates.count) updates"
   
    #initialize
    $updates=@()
    $results.updates | foreach {
        write-verbose $_.Title
     }
} #begin

Process {
    if ($private:all) {
        write-verbose "Including all updates"
        $updates+=$results.updates
    }
    elseif ($private:title) {
      foreach ($item in $title) {
          write-verbose "Getting title: $item"
          $updates+=$results.updates | where {$_.title -eq $item}
      } #foreach item
    }
    else {
        #otherwise get by severity
         Write-Verbose "Getting $severity updates"
        $updates+=$results.updates  | where {$_.msrcSeverity -eq $severity}  
    }
} #process

End {
    if ($updates) {
        write-verbose "Processing $($updates.count) updates"
        
        #download updates if needed
        $updates | where {-not $_.IsDownloaded} | foreach {
            write-verbose "Adding $($_.title) to download list"
            $updatesToDownload.Add($_) | out-null
        }
        if ($updatesToDownload.count -gt 0) {
            Write-Verbose "Downloading $($updatesToDownload.count) updates"
            $downloader.Updates = $updatesToDownload
            $downloader.Download()
        }
            
        Write-Verbose "Installing $($updates.count) updates"   
        foreach ($item in $updates) {
            $updatesToInstall.add($item) | out-Null
        }
        
        $installer.Updates = $updatesToInstall
        
        foreach ($update in $updates) {   
          
            #accept EULAs
            if ($update.EULAAccepted -eq $false) {
                $update.AcceptEULA=$True
            }
            
            #install the updates
            if ($pscmdlet.ShouldProcess($update.title)) {
              Write-Verbose "Installing $($update.title)"
              Write-Host "Installing $($update.title)" -ForegroundColor Green
              $installationResult = $installer.Install()
              
              #decode results
              Switch ($installationResult.ResultCode) {
                  0 {$result="Not Started"}
                  1 {$result="In Progress"}
                  2 {$result="Success" }
                  3 {$result="Success with errors"}
                  4 {$result="Failed"}
                  5 {$result="Process stopped bdefore completing" }
                  default {$result="Unknown"}
              } #switch
              
              #determine if any updates require a reboot
              if ($installationResult.RebootRequired) {
                $RebootRequired=$True
              }
              #create a custom result object
              New-Object PSObject -Property @{
                Computername=$env:computername
                Title=$update.title
                Result=$result
                RebootRequired=$installationResult.RebootRequired
                Severity=$update.msrcSeverity
                InstallDate=Get-Date
              } #new-object
            } #if should process
        } #end foreach
       
       if ($reboot -AND $rebootRequired) {
            Write-Verbose "Rebooting"
            if ($force) {
                Restart-Computer -force
            }
            else {
                Restart-Computer
            }
       }    
       elseif ($rebootRequired) {
            write-host "One or more updates requires a reboot." -ForegroundColor Red -BackgroundColor Black
       }
   } #if $updates
   else {
    Write-Warning "No matching updates found"
   }
   Write-Verbose "Ending $($myinvocation.mycommand)"
} #end

} #end Function


